/* ROOT(CINT) -*- C++ -*- link to some common Geant4 actions - implementation.
   Jacek M. Holeczek, 12.2002
   For details about ROOT and CINT see http://root.cern.ch */

#include "G4.hxx"

#include "Rtypes.h"
#include "TSystem.h"
#include "TApplication.h"

#include "G4ios.hh"
#include "G4RunManager.hh"
#include "G4UIsession.hh"
#include "G4UImanager.hh"

#if defined (G4UI_BUILD_ROOT_SESSION) || defined (G4UI_USE_ROOT)
#include "G4UIRoot.hh"
#else /* defined (G4UI_BUILD_ROOT_SESSION) || defined (G4UI_USE_ROOT) */
#include "G4UIterminal.hh"
#endif /* defined (G4UI_BUILD_ROOT_SESSION) || defined (G4UI_USE_ROOT) */

#include "G4VisManager.hh"

#include "G4GlobalFastSimulationManager.hh"

G4 *gG4 = (G4 *)0;

G4RunManager *G4::RunManager = (G4RunManager *)0;
G4UIsession  *G4::UIsession  = (G4UIsession *)0;
G4UImanager  *G4::UImanager  = (G4UImanager *)0;
G4VisManager *G4::VisManager = (G4VisManager *)0;

G4::G4()
{
  if (gG4) return;
  gG4 = this;
  if (!RunManager) RunManager = G4RunManager::GetRunManager();
  if (!RunManager) RunManager = (new G4RunManager());
  if (RunManager) UImanager = G4UImanager::GetUIpointer();
  if ((RunManager) && (UImanager) && (!UIsession)) UIsession = UImanager->GetSession();
  if ((RunManager) && (!VisManager)) VisManager = ((G4VisManager *) G4VVisManager::GetConcreteInstance());
}

G4::~G4()
{
  if (!gG4) return;
  gSystem->ProcessEvents();
  if (VisManager) {
    delete VisManager;
    VisManager = (G4VisManager *)0;
    gSystem->ProcessEvents();
  }
  DeleteG4UIsession();
  UImanager = (G4UImanager *)0;
  if (RunManager) {
    delete RunManager;
    RunManager = (G4RunManager *)0;
    gSystem->ProcessEvents();
  }
  gG4 = (G4 *)0;
}

void G4::DeleteG4(void)
{
  if (!gG4) return;
  delete gG4;
}

G4 *G4::NewG4(void)
{
  if (gG4) return gG4;
  return (new G4());
}

void G4::DeleteG4UIsession(G4UIsession *pointer)
{
  if (!gG4) return;
  gSystem->ProcessEvents();
  if (pointer) {
#if ROOT_VERSION_CODE > ROOT_VERSION(3,04,02)
    if (gApplication)
      {
	Bool_t OldRFR = gApplication->ReturnFromRun();
	gApplication->SetReturnFromRun(kFALSE);
	delete pointer;
	gApplication->SetReturnFromRun(OldRFR);
      }
    else
      delete pointer;
#else
    delete pointer;
#endif
    gSystem->ProcessEvents();
    if (pointer == UIsession)
      {
	UIsession = ((G4UIsession *)0);
	UImanager = G4UImanager::GetUIpointer();
	gSystem->ProcessEvents();
      }
  }
}

G4UIsession *G4::NewG4UIsession(int argc, char **argv)
{
  if ((RunManager) && (UIsession)) DeleteG4UIsession();
  else gSystem->ProcessEvents();
#if defined (G4UI_BUILD_ROOT_SESSION) || defined (G4UI_USE_ROOT)
  if ((RunManager) && (!UIsession)) UIsession = (new G4UIRoot((argc), (argv)));
#else /* defined (G4UI_BUILD_ROOT_SESSION) || defined (G4UI_USE_ROOT) */
  if ((RunManager) && (!UIsession))
    {
      G4cerr << "Info: G4UIRoot not available. Using G4UIterminal instead !!!"
	     << G4endl;
      G4cout << "Note: to return from G4UIterminal to root type \"exit\" !!!"
	     << G4endl;
      UIsession = (new G4UIterminal());
    }
#endif /* defined (G4UI_BUILD_ROOT_SESSION) || defined (G4UI_USE_ROOT) */
  gSystem->ProcessEvents();
  if (UIsession) UImanager = G4UImanager::GetUIpointer();
  return UIsession;
}

G4int G4::ApplyCommand(const char *aCommand)
{
  if (UImanager) return UImanager->ApplyCommand(aCommand);
  return -1;
}

G4int G4::ApplyCommand(G4String aCommand)
{
  if (UImanager) return UImanager->ApplyCommand(aCommand);
  return -1;
}

void G4::BeamOn(G4int n_event, const char *macroFile,
		G4int n_select)
{
  if (RunManager) RunManager->BeamOn(n_event, macroFile, n_select);
}

void G4::CloseFastSimulation(void)
{
  // closing the "fast simulation" will trigger
  // the ghost geometries construction
  G4GlobalFastSimulationManager::GetGlobalFastSimulationManager()->
    CloseFastSimulation();
}

void G4::ExecuteMacroFile(const char *fileName)
{
  if (UImanager) UImanager->ExecuteMacroFile(fileName);
}

void G4::Foreach(const char *macroFile, const char *variableName,
		 const char *candidates)
{
  if (UImanager) UImanager->Foreach(macroFile, variableName, candidates);
}

G4double G4::GetCurrentDoubleValue(const char *aCommand,
				   const char *aParameterName,
				   G4bool reGet)
{
  if (UImanager) return UImanager->GetCurrentDoubleValue(aCommand, aParameterName, reGet);
  return -1.0;
}

G4double G4::GetCurrentDoubleValue(const char *aCommand,
				   G4int parameterNumber,
				   G4bool reGet)
{
  if (UImanager) return UImanager->GetCurrentDoubleValue(aCommand, parameterNumber, reGet);
  return -1.0;
}

G4int G4::GetCurrentIntValue(const char *aCommand,
			     const char *aParameterName,
			     G4bool reGet)
{
  if (UImanager) return UImanager->GetCurrentIntValue(aCommand, aParameterName, reGet);
  return -1;
}

G4int G4::GetCurrentIntValue(const char *aCommand,
			     G4int parameterNumber,
			     G4bool reGet)
{
  if (UImanager) return UImanager->GetCurrentIntValue(aCommand, parameterNumber, reGet);
  return -1;
}

G4String G4::GetCurrentStringValue(const char *aCommand,
				   const char *aParameterName,
				   G4bool reGet)
{
  if (UImanager) return UImanager->GetCurrentStringValue(aCommand, aParameterName, reGet);
  return G4String("");
}

G4String G4::GetCurrentStringValue(const char *aCommand,
				   G4int parameterNumber,
				   G4bool reGet)
{
  if (UImanager) return UImanager->GetCurrentStringValue(aCommand, parameterNumber, reGet);
  return G4String("");
} 

G4String G4::GetCurrentValues(const char *aCommand)
{
  if (UImanager) return UImanager->GetCurrentValues(aCommand);
  return G4String("");
}

void G4::Initialise(void)
{
  gSystem->ProcessEvents();
  if (RunManager) {
    RunManager->Initialize();
    gSystem->ProcessEvents();
  }
}

void G4::Loop(const char *macroFile, const char *variableName,
	      G4double initialValue, G4double finalValue,
	      G4double stepSize)
{
  if (UImanager) UImanager->Loop(macroFile, variableName,
				 initialValue, finalValue,
				 stepSize);
}

G4UIsession *G4::SessionStart(int argc, char **argv)
{
  if ((RunManager) && (!UIsession)) NewG4UIsession(argc, argv);
  else gSystem->ProcessEvents();
  if (UIsession) {
    UIsession =  UIsession->SessionStart();
    if (UIsession) UImanager = G4UImanager::GetUIpointer();
    gSystem->ProcessEvents();
  }
  return UIsession;
}

void G4::SetVerboseLevel(G4int vl)
{
  if (RunManager) RunManager->SetVerboseLevel(vl);
  gSystem->ProcessEvents();
}

void G4::VisInitialise(void)
{
  gSystem->ProcessEvents();
  if (VisManager) {
    VisManager->Initialise();
    gSystem->ProcessEvents();
  }
}

void G4::VisSetVerboseLevel(G4int vl)
{
  if (VisManager) VisManager->SetVerboseLevel(vl);
  gSystem->ProcessEvents();
}

void G4::VisSetVerboseLevel(const G4String &vl)
{
  if (VisManager) VisManager->SetVerboseLevel(vl);
  gSystem->ProcessEvents();
}

/* "link C++" G4cin and G4endl */

static int G4_lib_init = ( G__load_text("std::istream &G4cin = std::cin; G__CINT_ENDL &G4endl = std::endl;"),
			   0 );

/* end of file G4.cxx by Jacek M. Holeczek */
