/* ROOT(CINT) -*- C++ -*- link to some common Geant4 actions - interface.
   Jacek M. Holeczek, 12.2002
   For details about ROOT and CINT see http://root.cern.ch */

#ifndef G4_HXX
#define G4_HXX

#if defined(__GNUC__)

#if !defined(GNU_GCC)
#define GNU_GCC 1
#endif /* !defined(GNU_GCC) */

#if (__GNUC__ > 2) || ( (__GNUC__ == 2) && (__GNUC_MINOR__ > 91) )
#if !defined(G4USE_STD_NAMESPACE)
#define G4USE_STD_NAMESPACE 1
#endif /* !defined(G4USE_STD_NAMESPACE) */
#endif /* (__GNUC__ > 2) || ( (__GNUC__ == 2) && (__GNUC_MINOR__ > 91) ) */

#endif /* defined(__GNUC__) */

#if !defined(G4VIS_NONE) && !defined(G4VIS_USE)
#define G4VIS_USE 1
#endif /* !defined(G4VIS_NONE) && !defined(G4VIS_USE) */

#if !defined(G4ANALYSIS_NONE) && !defined(G4ANALYSIS_USE_ROOT)
#define G4ANALYSIS_USE_ROOT 1
#endif /* !defined(G4ANALYSIS_NONE) && !defined(G4ANALYSIS_USE_ROOT) */

#if !defined (__CINT__) || defined (__MAKECINT__)
#include "Rtypes.h"
#endif /* !defined (__CINT__) || defined (__MAKECINT__) */

#include "TSystem.h"

#include "CLHEP/config/CLHEP.h"
#include "G4Types.hh"
#include "G4ios.hh"
#include "G4String.hh"
#include "Randomize.hh"
#include "CLHEP/Random/RanecuEngine.h"
#include "CLHEP/Random/RandomEngine.h"
#include "CLHEP/Random/RandGeneral.h"

/* backwards compatibility for CLHEP 1.9 and onwards */
namespace CLHEP {} using namespace CLHEP;

#if !defined (__CINT__) && !defined (__MAKECINT__)
#include "G4RunManager.hh"
#include "G4UIsession.hh"
#include "G4UImanager.hh"
#include "G4VisManager.hh"
#else /* !defined (__CINT__) && !defined (__MAKECINT__) */
class G4RunManager;
class G4UIsession;
class G4UImanager;
class G4VisManager;
#endif /* !defined (__CINT__) && !defined (__MAKECINT__) */

class G4
{
public:
  static G4RunManager *RunManager;
  static G4UIsession  *UIsession;
  static G4UImanager  *UImanager;
  static G4VisManager *VisManager;
  
public:
  G4();
  ~G4();
  
  static void DeleteG4(void);
  static G4 *NewG4(void);
  
  static void DeleteG4UIsession(G4UIsession *pointer = G4::UIsession);
  static G4UIsession *NewG4UIsession(int argc = 1, char **argv = 0);
  
  static G4int ApplyCommand(const char *aCommand);
  static G4int ApplyCommand(G4String aCommand);
  static void BeamOn(G4int n_event = 1, const char *macroFile = 0,
		     G4int n_select = -1);
  static void CloseFastSimulation(void);
  static void ExecuteMacroFile(const char *fileName);
  static void Foreach(const char *macroFile, const char *variableName,
		      const char *candidates);
  static G4double GetCurrentDoubleValue(const char *aCommand,
					const char *aParameterName,
					G4bool reGet = true);
  static G4double GetCurrentDoubleValue(const char *aCommand,
					G4int parameterNumber = 1,
					G4bool reGet = true);
  static G4int GetCurrentIntValue(const char *aCommand,
				  const char *aParameterName,
				  G4bool reGet = true);
  static G4int GetCurrentIntValue(const char *aCommand,
				  G4int parameterNumber = 1,
				  G4bool reGet = true);
  static G4String GetCurrentStringValue(const char *aCommand,
					const char *aParameterName,
					G4bool reGet = true);
  static G4String GetCurrentStringValue(const char *aCommand,
					G4int parameterNumber = 1,
					G4bool reGet = true);
  static G4String GetCurrentValues(const char *aCommand);
  static void Initialise(void);
  static void Initialize(void) { Initialise(); }
  static void Loop(const char *macroFile, const char *variableName,
		   G4double initialValue, G4double finalValue,
		   G4double stepSize = 1.0);
  static G4UIsession *SessionStart(int argc = 1, char **argv = 0);
  static void SetVerboseLevel(G4int vl);
  
  static void VisInitialise(void);
  static void VisInitialize(void) { VisInitialise(); }
  static void VisSetVerboseLevel(G4int vl);
  static void VisSetVerboseLevel(const G4String &vl);
  
};

R__EXTERN G4 *gG4;

#if !defined (__CINT__) || defined (__MAKECINT__)

/* ************************************************************************* */

/*
  The macro G4CINTCLASS(MyG4Class) generates :
  
  class MyG4Class;
  
  but ONLY if CINT / MAKECINT parse the code !!!
  
*/

#if defined (__CINT__) || defined (__MAKECINT__)
#define G4CINTCLASS(MyG4Class) class MyG4Class;
#else /* defined (__CINT__) || defined(__MAKECINT__) */
#define G4CINTCLASS(MyG4Class) /* */
#endif /* defined (__CINT__) || defined(__MAKECINT__) */

/* ************************************************************************* */

/*
  The macro G4VISMANAGER(MyVisManager) generates :
  
  inline void DeleteMyVisManager(MyVisManager *pointer);
  inline MyVisManager *NewMyVisManager(void);
  inline void SetMyVisManager(MyVisManager *pointer);
  inline void SetMyVisManager(void);
  
*/

#define G4VISMANAGER(MyVisManager) \
G4CINTCLASS(MyVisManager) \
inline void Delete##MyVisManager(MyVisManager *pointer = ((MyVisManager *)G4::VisManager)) \
{ \
gSystem->ProcessEvents(); \
if (pointer) { \
delete pointer; \
if (pointer == ((MyVisManager *)G4::VisManager)) G4::VisManager = ((G4VisManager *)0); \
gSystem->ProcessEvents(); \
} \
} \
inline MyVisManager *New##MyVisManager(void) \
{ \
MyVisManager *tmp = ((MyVisManager *)0); \
gSystem->ProcessEvents(); \
if (G4::RunManager) \
{ \
if (G4::VisManager) { \
delete G4::VisManager; \
gSystem->ProcessEvents(); \
} \
tmp = (new MyVisManager()); \
G4::VisManager = tmp; \
gSystem->ProcessEvents(); \
} \
return tmp; \
} \
inline void Set##MyVisManager(MyVisManager *pointer) \
{ \
if (G4::RunManager) G4::VisManager = pointer; \
gSystem->ProcessEvents(); \
} \
inline void Set##MyVisManager(void) \
{ \
gSystem->ProcessEvents(); \
if (G4::RunManager) \
{ \
if (G4::VisManager) { \
delete G4::VisManager; \
gSystem->ProcessEvents(); \
} \
G4::VisManager = (new MyVisManager()); \
gSystem->ProcessEvents(); \
} \
}

/* ************************************************************************* */

/*
  The macro G4DELETE(MyG4Class) generates :
  
  inline void DeleteMyG4Class(MyG4Class *pointer);
  
*/

#define G4DELETE(MyG4Class) \
G4CINTCLASS(MyG4Class) \
inline void Delete##MyG4Class(MyG4Class *pointer) \
{ \
if (pointer) delete pointer; \
}

/* ************************************************************************* */

/*
  The macro G4NEW(MyG4Class) generates :
  
  inline MyG4Class *NewMyG4Class(void);
  
*/

#define G4NEW(MyG4Class) \
G4CINTCLASS(MyG4Class) \
inline MyG4Class *New##MyG4Class(void) \
{ \
return (new MyG4Class()); \
}

/* ************************************************************************* */

/*
  The macro G4NEW1(MyG4Class,MyUserInit1) generates :
  
  inline MyG4Class *NewMyG4Class(MyUserInit1 *p1);
  
*/

#define G4NEW1(MyG4Class,MyUserInit1) \
G4CINTCLASS(MyG4Class) \
G4CINTCLASS(MyUserInit1) \
inline MyG4Class *New##MyG4Class(MyUserInit1 *p1) \
{ \
return (new MyG4Class((p1))); \
}

/* ************************************************************************* */

/*
  The macro G4NEW2(MyG4Class,MyUserInit1,MyUserInit2) generates :
  
  inline MyG4Class *NewMyG4Class(MyUserInit1 *p1,MyUserInit2 *p2);
  
*/

#define G4NEW2(MyG4Class,MyUserInit1,MyUserInit2) \
G4CINTCLASS(MyG4Class) \
G4CINTCLASS(MyUserInit1) \
G4CINTCLASS(MyUserInit2) \
inline MyG4Class *New##MyG4Class(MyUserInit1 *p1,MyUserInit2 *p2) \
{ \
return (new MyG4Class((p1),(p2))); \
}

/* ************************************************************************* */

/*
  The macro G4NEW3(MyG4Class,MyUserInit1,MyUserInit2,MyUserInit3) generates :
  
  inline MyG4Class *NewMyG4Class(MyUserInit1 *p1,MyUserInit2 *p2,MyUserInit3 *p3);
  
*/

#define G4NEW3(MyG4Class,MyUserInit1,MyUserInit2,MyUserInit3) \
G4CINTCLASS(MyG4Class) \
G4CINTCLASS(MyUserInit1) \
G4CINTCLASS(MyUserInit2) \
G4CINTCLASS(MyUserInit3) \
inline MyG4Class *New##MyG4Class(MyUserInit1 *p1,MyUserInit2 *p2,MyUserInit3 *p3) \
{ \
return (new MyG4Class((p1),(p2),(p3))); \
}

/* ************************************************************************* */

/*
  The macro G4CLASS(MyG4Class) generates :
  
  inline void DeleteMyG4Class(MyG4Class *pointer);
  inline MyG4Class *NewMyG4Class(void);
  
*/

#define G4CLASS(MyG4Class) \
G4DELETE(MyG4Class) \
G4NEW(MyG4Class)

/* ************************************************************************* */

/*
  The macro G4STEPPINGVERBOSE(MySteppingVerbose) generates :
  
  inline void DeleteMySteppingVerbose(MySteppingVerbose *pointer);
  inline MySteppingVerbose *NewMySteppingVerbose(void);
  inline void SetMySteppingVerbose(MySteppingVerbose *pointer);
  inline void SetMySteppingVerbose(void);
  
*/

#define G4STEPPINGVERBOSE(MySteppingVerbose) \
G4CLASS(MySteppingVerbose) \
inline void Set##MySteppingVerbose(MySteppingVerbose *pointer) \
{ \
G4VSteppingVerbose::SetInstance(pointer); \
} \
inline void Set##MySteppingVerbose(void) \
{ \
G4VSteppingVerbose::SetInstance((new MySteppingVerbose())); \
}

/* ************************************************************************* */

/*
  The macro G4USERINITIALIZATION(MyUserInitialization) generates :
  
  inline void DeleteMyUserInitialization(MyUserInitialization *pointer);
  inline MyUserInitialization *NewMyUserInitialization(void);
  inline void SetMyUserInitialization(MyUserInitialization *pointer);
  inline void SetMyUserInitialization(void);
  
*/

#define G4USERINITIALIZATION(MyUserInitialization) \
G4CLASS(MyUserInitialization) \
inline void Set##MyUserInitialization(MyUserInitialization *pointer) \
{ \
if (G4::RunManager) G4::RunManager->SetUserInitialization(pointer); \
} \
inline void Set##MyUserInitialization(void) \
{ \
if (G4::RunManager) G4::RunManager->SetUserInitialization((new MyUserInitialization())); \
}

/* ************************************************************************* */

/*
  The macro G4USERINITIALIZATION1(MyUserInitialization,MyUserInit1) generates :
  
  inline MyUserInitialization *NewMyUserInitialization(MyUserInit1 *p1);
  inline void SetMyUserInitialization(MyUserInit1 *p1);
  
*/

#define G4USERINITIALIZATION1(MyUserInitialization,MyUserInit1) \
G4NEW1(MyUserInitialization,MyUserInit1) \
inline void Set##MyUserInitialization(MyUserInit1 *p1) \
{ \
if (G4::RunManager) G4::RunManager->SetUserInitialization((new MyUserInitialization((p1)))); \
}

/* ************************************************************************* */

/*
  The macro G4USERINITIALIZATION2(MyUserInitialization,MyUserInit1,MyUserInit2) generates :
  
  inline MyUserInitialization *NewMyUserInitialization(MyUserInit1 *p1,MyUserInit2 *p2);
  inline void SetMyUserInitialization(MyUserInit1 *p1,MyUserInit2 *p2);
  
*/

#define G4USERINITIALIZATION2(MyUserInitialization,MyUserInit1,MyUserInit2) \
G4NEW2(MyUserInitialization,MyUserInit1,MyUserInit2) \
inline void Set##MyUserInitialization(MyUserInit1 *p1,MyUserInit2 *p2) \
{ \
if (G4::RunManager) G4::RunManager->SetUserInitialization((new MyUserInitialization((p1),(p2)))); \
}

/* ************************************************************************* */

/*
  The macro G4USERINITIALIZATION3(MyUserInitialization,MyUserInit1,MyUserInit2,MyUserInit3) generates :
  
  inline MyUserInitialization *NewMyUserInitialization(MyUserInit1 *p1,MyUserInit2 *p2,MyUserInit3 *p3);
  inline void SetMyUserInitialization(MyUserInit1 *p1,MyUserInit2 *p2,MyUserInit3 *p3);
  
*/

#define G4USERINITIALIZATION3(MyUserInitialization,MyUserInit1,MyUserInit2,MyUserInit3) \
G4NEW3(MyUserInitialization,MyUserInit1,MyUserInit2,MyUserInit3) \
inline void Set##MyUserInitialization(MyUserInit1 *p1,MyUserInit2 *p2,MyUserInit3 *p3) \
{ \
if (G4::RunManager) G4::RunManager->SetUserInitialization((new MyUserInitialization((p1),(p2),(p3)))); \
}

/* ************************************************************************* */

/*
  The macro G4USERACTION(MyUserAction) generates :
  
  inline void DeleteMyUserAction(MyUserAction *pointer);
  inline MyUserAction *NewMyUserAction(void);
  inline void SetMyUserAction(MyUserAction *pointer);
  inline void SetMyUserAction(void);
  
*/

#define G4USERACTION(MyUserAction) \
G4CLASS(MyUserAction) \
inline void Set##MyUserAction(MyUserAction *pointer) \
{ \
if (G4::RunManager) G4::RunManager->SetUserAction(pointer); \
} \
inline void Set##MyUserAction(void) \
{ \
if (G4::RunManager) G4::RunManager->SetUserAction((new MyUserAction())); \
}

/* ************************************************************************* */

/*
  The macro G4USERACTION1(MyUserAction,MyUserInit1) generates :
  
  inline MyUserAction *NewMyUserAction(MyUserInit1 *p1);
  inline void SetMyUserAction(MyUserInit1 *p1);
  
*/

#define G4USERACTION1(MyUserAction,MyUserInit1) \
G4NEW1(MyUserAction,MyUserInit1) \
inline void Set##MyUserAction(MyUserInit1 *p1) \
{ \
if (G4::RunManager) G4::RunManager->SetUserAction((new MyUserAction((p1)))); \
}

/* ************************************************************************* */

/*
  The macro G4USERACTION2(MyUserAction,MyUserInit1,MyUserInit2) generates :
  
  inline MyUserAction *NewMyUserAction(MyUserInit1 *p1,MyUserInit2 *p2);
  inline void SetMyUserAction(MyUserInit1 *p1,MyUserInit2 *p2);
  
*/

#define G4USERACTION2(MyUserAction,MyUserInit1,MyUserInit2) \
G4NEW2(MyUserAction,MyUserInit1,MyUserInit2) \
inline void Set##MyUserAction(MyUserInit1 *p1,MyUserInit2 *p2) \
{ \
if (G4::RunManager) G4::RunManager->SetUserAction((new MyUserAction((p1),(p2)))); \
}

/* ************************************************************************* */

/*
  The macro G4USERACTION3(MyUserAction,MyUserInit1,MyUserInit2,MyUserInit3) generates :
  
  inline MyUserAction *NewMyUserAction(MyUserInit1 *p1,MyUserInit2 *p2,MyUserInit3 *p3);
  inline void SetMyUserAction(MyUserInit1 *p1,MyUserInit2 *p2,MyUserInit3 *p3);
  
*/

#define G4USERACTION3(MyUserAction,MyUserInit1,MyUserInit2,MyUserInit3) \
G4NEW3(MyUserAction,MyUserInit1,MyUserInit2,MyUserInit3) \
inline void Set##MyUserAction(MyUserInit1 *p1,MyUserInit2 *p2,MyUserInit3 *p3) \
{ \
if (G4::RunManager) G4::RunManager->SetUserAction((new MyUserAction((p1),(p2),(p3)))); \
}

/* ************************************************************************* */

/*
  The macro G4SETVERBOSELEVEL(MyG4Class) generates :
  
  inline void MyG4ClassSetVerboseLevel(MyG4Class *pointer, G4int vl);
  
*/

#define G4SETVERBOSELEVEL(MyG4Class) \
G4CINTCLASS(MyG4Class) \
inline void MyG4Class##SetVerboseLevel(MyG4Class *pointer, G4int vl) \
{ \
if (pointer) pointer->SetVerboseLevel(vl); \
gSystem->ProcessEvents(); \
}

/* ************************************************************************* */

/*
  The macro G4SETDRAWFLAG(MyG4Class) generates :
  
  inline void MyG4ClassSetDrawFlag(MyG4Class *pointer, G4String df);
  
*/

#define G4SETDRAWFLAG(MyG4Class) \
G4CINTCLASS(MyG4Class) \
inline void MyG4Class##SetDrawFlag(MyG4Class *pointer, G4String df) \
{ \
if (pointer) pointer->SetDrawFlag(df); \
gSystem->ProcessEvents(); \
}

/* ************************************************************************* */

#endif /* !defined (__CINT__) || defined (__MAKECINT__) */

#endif /* ifndef G4_HXX */

/* end of file G4.hxx by Jacek M. Holeczek */
