#!/bin/sh

# In order to "add" Geant4 support to ROOT, take the following steps :
#  1. "export ROOTSYS=/where/your/ROOT/distribution/resides"
#  2. "export PATH=$ROOTSYS/bin:$PATH"
#  3. "export LD_LIBRARY_PATH=$ROOTSYS/lib:$LD_LIBRARY_PATH"
#  4. ". /where/your/Geant4/distribution/resides/env.sh"
#  5. ... if you have a "Geant4 specific" OpenGL / Mesa ... use one of ...
#     "export LD_LIBRARY_PATH=$OGLHOME/lib:$LD_LIBRARY_PATH"
#     "export LD_LIBRARY_PATH=$OGLHOME/lib/$G4SYSTEM:$LD_LIBRARY_PATH"
#     "export LD_LIBRARY_PATH=/where/your/GL/libs/reside:$LD_LIBRARY_PATH"
#  6. "export LD_LIBRARY_PATH=$G4LIB/$G4SYSTEM:$CLHEP_LIB_DIR:$LD_LIBRARY_PATH"
#  7. ... if you have "Geant4 educated guess physics lists" ... use one of ...
#     "export LD_LIBRARY_PATH=$G4LIB/plists/$G4SYSTEM:$LD_LIBRARY_PATH"
#     "export LD_LIBRARY_PATH=$G4LISTS_BASE/hadronic/plists/lib/$G4SYSTEM:$LD_LIBRARY_PATH"
#     "export LD_LIBRARY_PATH=/where/your/plists/libs/reside:$LD_LIBRARY_PATH"
#  8. run this script (prepared for linux) "sh make.G4"

#set -x

echo "... building the lists of libraries ..."

export CLHEP_LIBS="-L${CLHEP_LIB_DIR} -l${CLHEP_LIB}"

#export G4_LIBS="-L${G4LIB}/${G4SYSTEM} ${G4LIB}/${G4SYSTEM}/libG[34]*.so"
export G4_LIBS="-L${G4LIB}/${G4SYSTEM} `(cd ${G4LIB}/${G4SYSTEM}; ls libG[34]*.so) | sed s/\^libG/-lG/g - | sed s/[.]so\$//g -`"
#export G4_LIBS="-L${G4LIB}/${G4SYSTEM} `${G4LIB}/${G4SYSTEM}/liblist -m ${G4LIB}/${G4SYSTEM} < ${G4LIB}/${G4SYSTEM}/libname.map`"
#export G4_LIBS="-L${G4LIB}/${G4SYSTEM} `${G4LIB}/${G4SYSTEM}/liblist -d ${G4INSTALL}/tmp/${G4SYSTEM} < ${G4LIB}/${G4SYSTEM}/libname.map`"

#CLHEP_LIBS="-Wl,-rpath,${CLHEP_LIB_DIR} ${CLHEP_LIBS}"
#G4_LIBS="-Wl,-rpath,${G4LIB}/${G4SYSTEM} ${G4_LIBS}"

export X11_LIB_DIR="/usr/X11R6/lib"

export GL_INC=""
if [ "X${OGLFLAGS}" != "X" ] ; then
    GL_INC="${OGLFLAGS}"
else
    if [ "X${OGLHOME}" != "X" ] ; then
	GL_INC="-I${OGLHOME}/include"
    fi
fi

export GL_LIBS="-lGLU -lGL"
if [ "X${OGLLIBS}" != "X" ] ; then
    GL_LIBS="${OGLLIBS}"
else
    if [ "X${OGLHOME}" != "X" ] ; then
	GL_LIBS="-L${OGLHOME}/lib/${G4SYSTEM} -L${OGLHOME}/lib ${GL_LIBS}"
    else
	GL_LIBS="-L${X11_LIB_DIR} -L/usr/lib -L/lib ${GL_LIBS}"
    fi
fi

export X11_LIBS="-L${X11_LIB_DIR} -L/usr/lib -L/lib -lXmu -lXt -lXext -lX11 -lXi -lSM -lICE" # X11
X11_LIBS="${X11_LIBS} -lXaw" # Xaw
X11_LIBS="${X11_LIBS} -lXm -lXp -lXpm" # Motif / LessTif + XPM
#X11_LIBS="${X11_LIBS} -lGLU -lGL" # "system default" OpenGL / Mesa
X11_LIBS="${GL_LIBS} ${X11_LIBS}" # "Geant4 specific" OpenGL / Mesa
#export X11_LIBS="-L/lib -L/usr/lib -L${X11_LIB_DIR} -Wl,-rpath,${X11_LIB_DIR} ${X11_LIB_DIR}/*.so"
#export X11_LIBS="-L/lib -L/usr/lib -L${X11_LIB_DIR} -lXt -lXaw -lXp -lXm -lGLU -lGL" # a minimal set
# Note that instead of linking with all these libraries above you could also :
#  root [0] gSystem->Load("/usr/X11R6/lib/libXt")
#  root [1] gSystem->Load("/usr/X11R6/lib/libXaw")
#  root [2] gSystem->Load("/usr/X11R6/lib/libXp")
#  root [3] gSystem->Load("/usr/X11R6/lib/libXm")
#  root [4] gSystem->Load(gSystem->ExpandPathName("${OGLHOME}/lib/libGLU"))
#  root [5] gSystem->Load(gSystem->ExpandPathName("${OGLHOME}/lib/libGL"))

if [ "X${CXX}" = "X" ] ; then
    export CXX=g++
    if [ "X${CXXFLAGS}" = "X" ] ; then
	export CXXFLAGS="-g -O2"
    fi
fi

ROOTCFLAGS="`${ROOTSYS}/bin/root-config --cflags`"
#ROOTCFLAGS="`${ROOTSYS}/bin/root-config --cflags | sed 's/\(-D_REENTRANT \|-pthread \)//g'`"
ROOTLIBS="`${ROOTSYS}/bin/root-config --nonew --glibs | sed 's/\(-lNew \)//g'`"
#ROOTLIBS="`${ROOTSYS}/bin/root-config --nonew --glibs | sed 's/\(-lNew \|-lThread \|-lpthread \|-pthread \)//g'`"

GEANT4_FLAGS=""

if [ "X${G4UI_BUILD_ROOT_SESSION}" != "X" ] ; then
    GEANT4_FLAGS="${GEANT4_FLAGS} -DG4UI_BUILD_ROOT_SESSION=${G4UI_BUILD_ROOT_SESSION}"
fi

if [ "X${G4UI_USE_ROOT}" != "X" ] ; then
    GEANT4_FLAGS="${GEANT4_FLAGS} -DG4UI_USE_ROOT=${G4UI_USE_ROOT}"
fi

GEANT4_FIX_FLAGS=""
#GEANT4_FIX_FLAGS="${GEANT4_FIX_FLAGS} -fno-access-control"

echo "... building the G__G4.* CINT dictionary ..."

rm -f G__G4.*
rootcint -f G__G4.cxx -c -p ${CPPFLAGS} ${GEANT4_FLAGS} -I${G4INCLUDE} -I${CLHEP_INCLUDE_DIR} G4.hxx G4LinkDef.h
#rootcint -f G__G4.cxx -c -p ${CPPFLAGS} ${GEANT4_FLAGS} -I${G4INCLUDE} -I${CLHEP_INCLUDE_DIR} ${GL_INC} G4.hxx G4LinkDef.h
${CXX} ${CPPFLAGS} ${CXXFLAGS} -Wall -fPIC -I. ${GEANT4_FLAGS} ${GEANT4_FIX_FLAGS} ${ROOTCFLAGS} -I${G4INCLUDE} -I${CLHEP_INCLUDE_DIR} ${GL_INC} -c G__G4.cxx

echo "... building the G4.so library ..."

rm -f G4.so
${CXX} ${CPPFLAGS} ${CXXFLAGS} -Wall -fPIC -I. ${GEANT4_FLAGS} ${GEANT4_FIX_FLAGS} ${ROOTCFLAGS} -I${G4INCLUDE} -I${CLHEP_INCLUDE_DIR} ${GL_INC} -shared -o G4.so G4.cxx G__G4.o ${ROOTLIBS} ${G4_LIBS} ${CLHEP_LIBS} ${X11_LIBS}
strip G4.so

echo "... copying the G4.so and G4.hxx to global destinations ..."

if [ ! -d ${G4WORKDIR}/lib/${G4SYSTEM} ] ; then
    mkdir -p ${G4WORKDIR}/lib/${G4SYSTEM}
fi

if [ ! -d ${G4WORKDIR}/include ] ; then
    mkdir -p ${G4WORKDIR}/include
fi

cp -a G4.so ${G4WORKDIR}/lib/${G4SYSTEM}/
cp -a G4.hxx ${G4WORKDIR}/include/

#cp -a G4.so ${G4LIB}/${G4SYSTEM}/
#cp -a G4.hxx ${G4INCLUDE}/

#cp -a G4.so ${ROOTSYS}/lib/
#cp -a G4.hxx ${ROOTSYS}/include/

echo "... done ..."

# end of file make.G4 by Jacek M. Holeczek
