//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//

#ifndef IcaAnalysisExample_h
#define IcaAnalysisExample_h 1

#include "IcaAnalysisManager.hh"

class G4VPhysicalVolume;
class G4Event;
class G4Run;
class G4Track;
class G4Step;

#include "G4ClassificationOfNewTrack.hh"
#include "G4TrackStatus.hh"
#include "G4Types.hh"

#include <time.h>

#if defined (G4UI_USE_ROOT) || defined (G4UI_BUILD_ROOT_SESSION)
#if !defined (G4ANALYSIS_USE_ROOT)
#define G4ANALYSIS_USE_ROOT 1
#endif /* !defined (G4ANALYSIS_USE_ROOT) */
#endif /* defined (G4UI_USE_ROOT) || defined (G4UI_BUILD_ROOT_SESSION) */

#if defined (G4ANALYSIS_USE_ROOT)

class TH1D;

#endif /* defined (G4ANALYSIS_USE_ROOT) */

class IcaAnalysisExample : public IcaAnalysisManager {
  
public:
  IcaAnalysisExample();
  ~IcaAnalysisExample();
  
public:
  // G4VUserDetectorConstruction
  void Construct(const G4VPhysicalVolume*);
  
  // G4VUserPhysicsList
  void ConstructParticle();
  void ConstructProcess();
  void SetCuts();
  
  // G4VUserPrimaryGeneratorAction
  void GeneratePrimaries(const G4Event*);
  
  // G4UserRunAction
  void BeginOfRunAction(const G4Run*);
  void EndOfRunAction(const G4Run*);
  
  // G4UserEventAction
  void BeginOfEventAction(const G4Event*);
  void EndOfEventAction(const G4Event*);
  
  // G4UserStackingAction
  void ClassifyNewTrack(const G4Track*, G4ClassificationOfNewTrack*);
  void NewStage();
  void PrepareNewEvent();
  
  // G4UserTrackingAction
  void PreUserTrackingAction(const G4Track*);
  void PostUserTrackingAction(const G4Track*, G4TrackStatus*);
  
  // G4UserSteppingAction
  void UserSteppingAction(const G4Step*);
  
  // once a while do "something"
  void OnceAWhileDoIt(const G4bool DoItNow = false);
  
private:
  
  time_t LastDoItTime; // used in OnceAWhileDoIt method
  
#if defined (G4ANALYSIS_USE_ROOT)
  
  TH1D *hStepLength; // Step Length
  TH1D *hStepTotELoss; // Step Total Energy Loss
  TH1D *hTotELossNorm; // Total Energy Loss per Step Length Unit
  
  TH1D *hOPWaveLength; // Wavelength of the produced optical photon
  
#endif /* defined (G4ANALYSIS_USE_ROOT) */
  
};

#endif

// end of file IcaAnalysisExample.hh by Jacek M. Holeczek
