//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#ifndef IcaDetectorConstruction_h
#define IcaDetectorConstruction_h 1

#include "G4VUserDetectorConstruction.hh"
#include "globals.hh"

class G4VPhysicalVolume;
class G4LogicalVolume;
class G4Material;
class IcaDetectorConstructionMessenger;
class G4UserLimits;

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

class IcaDetectorConstruction : public G4VUserDetectorConstruction
{
  public:
  
    IcaDetectorConstruction();
   ~IcaDetectorConstruction();

  public:
  
     G4VPhysicalVolume* Construct();
     
     const G4VPhysicalVolume* GetWorld()      { return pWorld; };
     G4double    GetWorldSizeX()        { return WorldSizeX; };
     G4double    GetWorldSizeY()        { return WorldSizeY; };
     G4double    GetWorldSizeZ()        { return WorldSizeZ; };
     G4Material* GetAbsorberMaterial()  { return dMaterial; };
     
     void        SetLArAbsLength(const G4double newValue) { LArAbsLength = newValue; };
     G4double    GetLArAbsLength()      { return LArAbsLength; };
     
     // methods for UserLimits in detector
     void      ResetUserLimits();
     void      UseUserLimits(G4bool value);
     G4bool    IsUseUserLimits() { return fUseUserLimits; }
     void      SetMaxTimeInDetector(G4double value);
     void      SetMinEkineInDetector(G4double value);
     void      SetMinRangeInDetector(G4double value);
     G4double  GetMaxTimeInDetector() const  { return theMaxTimeCutsInDetector;  }
     G4double  GetMinEkineInDetector() const { return theMinEkineCutsInDetector; }
     G4double  GetMinRangeInDetector() const { return theMinRangeCutsInDetector; }
     
  private:
  
     G4double              WorldSizeX, WorldSizeY, WorldSizeZ;
     G4VPhysicalVolume*    pWorld;
     G4LogicalVolume*      lWorld;
     
     G4VPhysicalVolume*    pBox;
     G4LogicalVolume*      lBox;
     
     G4double              LArAbsLength;
     
     G4Material*           dMaterial;     
     G4Material*           wMaterial;     
     
     G4bool           fUseUserLimits;
     G4UserLimits*    theUserLimitsForDetector;
     G4double         theMaxTimeCutsInDetector;
     G4double         theMinEkineCutsInDetector;
     G4double         theMinRangeCutsInDetector;
     
     IcaDetectorConstructionMessenger* detectorMessenger;
     
  private:
    
     void               DefineMaterials();
     G4VPhysicalVolume* ConstructVolumes();     
};

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#endif

// end of file IcaDetectorConstruction.hh by Jacek M. Holeczek
