//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * The following disclaimer summarizes all the specific disclaimers *
// * of contributors to this software. The specific disclaimers,which *
// * govern, are listed with their locations in:                      *
// *   http://cern.ch/geant4/license                                  *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * This  code  implementation is the  intellectual property  of the *
// * GEANT4 collaboration.                                            *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//
// $Id: IcaPhysicsList.hh,v 1.7 2002/01/09 17:24:18 ranjard Exp $
// GEANT4 tag $Name: geant4-04-00-patch-02 $
//
// 

#ifndef IcaPhysicsList_h
#define IcaPhysicsList_h 1

#include "G4VUserPhysicsList.hh"
#include "globals.hh"

class IcaDetectorConstruction;
class IcaPhysicsListMessenger;
class IcaStepCut;
class G4Cerenkov;
class G4Scintillation;

class IcaPhysicsList: public G4VUserPhysicsList
{
public:
  IcaPhysicsList(IcaDetectorConstruction* = 0);
  virtual ~IcaPhysicsList();
  
protected:
  // Construct particle and physics
  virtual void ConstructParticle();
  virtual void ConstructProcess();
  
  virtual void SetCuts();
  
protected:
  // these methods Construct particles 
  virtual void ConstructBosons();
  virtual void ConstructLeptons();
  virtual void ConstructMesons();
  virtual void ConstructBaryons();
  virtual void ConstructIons();
  
protected:
  // these methods Construct physics processes and register them
  virtual void AddTransportation();
  virtual void AddParameterisation();
  virtual void ConstructGeneral();
  virtual void ConstructEM();
  virtual void ConstructOp();
  virtual void AddMaxChargedStepCut();
  
  public:

    void SetGammaCut(G4double);
    void SetElectronCut(G4double);
    void SetMuonCut(G4double);
    void SetProtonCut(G4double);
    void SetOpticalPhotonCut(G4double);

    void SetCutsByEnergy(G4double);

    void SetMaxStep(G4double);

    void GetRange(G4double);

    void RestoreDefaultCuts(G4double);

    void SetMaxNumOfCerenkovPhotonsPerStep(G4int);

    void SetScintillationYieldFactor(G4double);
    G4double GetScintillationYieldFactor() { return ScintillationYieldFactor; }

  public:

    G4double MaxChargedStep;

  private:

    IcaStepCut* theeminusStepCut;
    IcaStepCut* theeplusStepCut;
    IcaStepCut* themuStepCut;
    IcaStepCut* thehadronStepCut;

    G4double cutForGamma;
    G4double cutForElectron;
    G4double cutForMuon;
    G4double cutForProton;
    G4double cutForOpticalPhoton;
    G4double currentDefaultCut;

    G4Cerenkov*   theCerenkovProcess;

    G4int MaxNumOfCerenkovPhotonsPerStep;

    G4Scintillation* theScintillationProcess;

    G4double ScintillationYieldFactor;

    IcaDetectorConstruction* pDet;
    IcaPhysicsListMessenger* physicsListMessenger;
};

#endif

// end of file IcaPhysicsList.hh by Jacek M. Holeczek
