//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * The following disclaimer summarizes all the specific disclaimers *
// * of contributors to this software. The specific disclaimers,which *
// * govern, are listed with their locations in:                      *
// *   http://cern.ch/geant4/license                                  *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * This  code  implementation is the  intellectual property  of the *
// * GEANT4 collaboration.                                            *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//
// $Id: IcaStepCut.hh,v 1.2 2001/07/11 09:57:21 gunter Exp $
// GEANT4 tag $Name: geant4-04-00 $
//
// 

#ifndef IcaStepCut_h
#define IcaStepCut_h 1

#include "G4ios.hh"
#include "globals.hh"
#include "G4VDiscreteProcess.hh"
#include "G4Step.hh"
#include "G4Track.hh"

class IcaStepCut : public G4VDiscreteProcess
{
  public:     

     IcaStepCut(const G4String& processName ="UserStepCut" );
     IcaStepCut(IcaStepCut &);

     ~IcaStepCut();

     G4double PostStepGetPhysicalInteractionLength(
                             const G4Track& track,
			     G4double   previousStepSize,
			     G4ForceCondition* condition
			    );

     G4VParticleChange* PostStepDoIt(
			     const G4Track& ,
			     const G4Step& 
			    );

    void SetMaxStep(G4double);

  protected:

     // it is not needed here !
     G4double GetMeanFreePath(const G4Track& aTrack,
                             G4double   previousStepSize,
                             G4ForceCondition* condition
                            );

			    
  private:
  
  // hide assignment operator as private 
      IcaStepCut & operator=(const IcaStepCut &right);

  private:

     G4double MaxChargedStep ;
};

// inlined function members implementation

#include "G4UserLimits.hh"
#include "G4VParticleChange.hh"
#include "G4EnergyLossTables.hh"

inline G4VParticleChange* IcaStepCut::PostStepDoIt(
                             const G4Track& aTrack,
                             const G4Step&
                            )
{
   // do nothing
   aParticleChange.Initialize(aTrack);
   return &aParticleChange;
}

inline G4double IcaStepCut::GetMeanFreePath(const G4Track& /*aTrack*/,
                             G4double /*previousStepSize*/,
                             G4ForceCondition* /*condition*/
                            )
{
  return 0.;
}

#endif

// end of file IcaStepCut.hh by Jacek M. Holeczek
