//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

#include "IcaAnalysisExample.hh"

#include "G4ios.hh"

#include "G4RunManager.hh"
#include "G4VPhysicalVolume.hh"
#include "G4Event.hh"
#include "G4Run.hh"
#include "G4Track.hh"
#include "G4ClassificationOfNewTrack.hh"
#include "G4TrackStatus.hh"
#include "G4Step.hh"
#include "G4Types.hh"

#include <time.h>

static const G4double LambdaE = 2.0 * 3.14159265358979323846 * 1.973269602e-16 * m * GeV;

#if defined (G4ANALYSIS_USE_ROOT)

#include "TROOT.h"
#include "TApplication.h"
#include "TSystem.h"
#include "TH1.h"
#include "TPad.h"
#include "TCanvas.h"

#endif /* defined (G4ANALYSIS_USE_ROOT) */

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

IcaAnalysisExample::IcaAnalysisExample()
{
  LastDoItTime = (time_t)0;
  
#if defined (G4ANALYSIS_USE_ROOT)
  
  if (gSystem) gSystem->ProcessEvents();
  
  hStepLength = (TH1D *)0; // Step Length
  hStepTotELoss = (TH1D *)0; // Step Total Energy Loss
  hTotELossNorm = (TH1D *)0; // Total Energy Loss per Step Length Unit
  
  hOPWaveLength = (TH1D *)0; // Wavelength of the produced optical photon
  
#endif /* defined (G4ANALYSIS_USE_ROOT) */
  
  OnceAWhileDoIt();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

IcaAnalysisExample::~IcaAnalysisExample()
{
  //OnceAWhileDoIt();
  
#if defined (G4ANALYSIS_USE_ROOT)
  
  if (gSystem) gSystem->ProcessEvents();
  
  
  
#endif /* defined (G4ANALYSIS_USE_ROOT) */
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void IcaAnalysisExample::Construct(const G4VPhysicalVolume */*theWorldWolume*/)
{
#if defined (G4ANALYSIS_USE_ROOT)
  if (gSystem) gSystem->ProcessEvents();
  
  
  
#endif /* defined (G4ANALYSIS_USE_ROOT) */
  
  OnceAWhileDoIt();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void IcaAnalysisExample::ConstructParticle()
{
#if defined (G4ANALYSIS_USE_ROOT)
  
  if (gSystem) gSystem->ProcessEvents();
  
  
  
#endif /* defined (G4ANALYSIS_USE_ROOT) */
  
  OnceAWhileDoIt();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void IcaAnalysisExample::ConstructProcess()
{
#if defined (G4ANALYSIS_USE_ROOT)
  
  if (gSystem) gSystem->ProcessEvents();
  
  
  
#endif /* defined (G4ANALYSIS_USE_ROOT) */
  
  OnceAWhileDoIt();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void IcaAnalysisExample::SetCuts()
{
#if defined (G4ANALYSIS_USE_ROOT)
  
  if (gSystem) gSystem->ProcessEvents();
  
  
  
#endif /* defined (G4ANALYSIS_USE_ROOT) */
  
  OnceAWhileDoIt();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void IcaAnalysisExample::GeneratePrimaries(const G4Event */*anEvent*/)
{
#if defined (G4ANALYSIS_USE_ROOT)
  if (gSystem) gSystem->ProcessEvents();
  
  
  
#endif /* defined (G4ANALYSIS_USE_ROOT) */
  
  OnceAWhileDoIt();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void IcaAnalysisExample::BeginOfRunAction(const G4Run */*aRun*/)
{
#if defined (G4ANALYSIS_USE_ROOT)
  if (gSystem) gSystem->ProcessEvents();
  
  // Step Length
  hStepLength = (TH1D *)gROOT->FindObject("hStepLength");
  if (hStepLength) hStepLength->Reset();
  else
    {
      hStepLength = new TH1D("hStepLength","Step Length", 100000, 0.0, 100.0); // in [cm]
      if (hStepLength) hStepLength->SetXTitle("[cm]");
    }
  
  // Step Total Energy Loss
  hStepTotELoss = (TH1D *)gROOT->FindObject("hStepTotELoss");
  if (hStepTotELoss) hStepTotELoss->Reset();
  else
    {
      hStepTotELoss = new TH1D("hStepTotELoss","Step Total Energy Loss", 50000, 0.0, 50.0); // in [MeV]
      if (hStepTotELoss) hStepTotELoss->SetXTitle("[MeV]");
    }
  
  // Total Energy Loss per Step Length Unit
  hTotELossNorm = (TH1D *)gROOT->FindObject("hTotELossNorm");
  if (hTotELossNorm) hTotELossNorm->Reset();
  else
    {
      hTotELossNorm = new TH1D("hTotELossNorm","Total Energy Loss per Step Length Unit", 50000, 0.0, 50.0); // in [MeV/cm]
      if (hTotELossNorm) hTotELossNorm->SetXTitle("[MeV/cm]");
    }
  
  // Wavelength of the produced optical photon
  hOPWaveLength = (TH1D *)gROOT->FindObject("hOPWaveLength");
  if (hOPWaveLength) hOPWaveLength->Reset();
  else
    {
      hOPWaveLength = new TH1D("hOPWaveLength","Wavelength of the produced optical photon", 10000, 0.0, 1000.0); // in [nanometers]
      if (hOPWaveLength) hOPWaveLength->SetXTitle("[nm]");
    }
  
#endif /* defined (G4ANALYSIS_USE_ROOT) */
  
  OnceAWhileDoIt(true); // do it now
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void IcaAnalysisExample::EndOfRunAction(const G4Run */*aRun*/)
{
#if defined (G4ANALYSIS_USE_ROOT)
  if (gSystem) gSystem->ProcessEvents();
  
  
  
#endif /* defined (G4ANALYSIS_USE_ROOT) */
  
  OnceAWhileDoIt(true); // do it now
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void IcaAnalysisExample::BeginOfEventAction(const G4Event */*anEvent*/)
{
#if defined (G4ANALYSIS_USE_ROOT)
  if (gSystem) gSystem->ProcessEvents();
  
  
  
#endif /* defined (G4ANALYSIS_USE_ROOT) */
  
  OnceAWhileDoIt();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void IcaAnalysisExample::EndOfEventAction(const G4Event */*anEvent*/)
{
#if defined (G4ANALYSIS_USE_ROOT)
  if (gSystem) gSystem->ProcessEvents();
  
  
  
#endif /* defined (G4ANALYSIS_USE_ROOT) */
  
  OnceAWhileDoIt();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void IcaAnalysisExample::ClassifyNewTrack(
			     const G4Track *aTrack,
			     G4ClassificationOfNewTrack */*classification_ptr*/)
{
  // G4ClassificationOfNewTrack &classification = (*classification_ptr);
  
#if defined (G4ANALYSIS_USE_ROOT)
  
  if (gSystem) gSystem->ProcessEvents();
  
  G4String particleName = aTrack->GetDefinition()->GetParticleName();
  
  if (particleName == "opticalphoton")
    {
      Double_t aWaveLength = 0.0; // will be in [nanometer]
      aWaveLength = (LambdaE / aTrack->GetTotalEnergy()) / nanometer; // in [nanometer]
      // aWaveLength = (LambdaE / aTrack->GetKineticEnergy()) / nanometer; // in [nanometer]
      if (hOPWaveLength) hOPWaveLength->Fill(aWaveLength, aTrack->GetWeight()); // Wavelength of the produced optical photon
    }
  
#endif /* defined (G4ANALYSIS_USE_ROOT) */
  
  OnceAWhileDoIt();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void IcaAnalysisExample::NewStage()
{
#if defined (G4ANALYSIS_USE_ROOT)
  
  if (gSystem) gSystem->ProcessEvents();
  
  
  
#endif /* defined (G4ANALYSIS_USE_ROOT) */
  
  OnceAWhileDoIt();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void IcaAnalysisExample::PrepareNewEvent()
{
#if defined (G4ANALYSIS_USE_ROOT)
  
  if (gSystem) gSystem->ProcessEvents();
  
  
  
#endif /* defined (G4ANALYSIS_USE_ROOT) */
  
  OnceAWhileDoIt();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void IcaAnalysisExample::PreUserTrackingAction(const G4Track */*aTrack*/)
{
#if defined (G4ANALYSIS_USE_ROOT)
  if (gSystem) gSystem->ProcessEvents();
  
  
  
#endif /* defined (G4ANALYSIS_USE_ROOT) */
  
  OnceAWhileDoIt();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void IcaAnalysisExample::PostUserTrackingAction(const G4Track */*aTrack*/,
						G4TrackStatus */*status_ptr*/)
{
  // G4TrackStatus &status = (*status_ptr);
  
#if defined (G4ANALYSIS_USE_ROOT)
  if (gSystem) gSystem->ProcessEvents();
  
  
  
#endif /* defined (G4ANALYSIS_USE_ROOT) */
  
  OnceAWhileDoIt();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void IcaAnalysisExample::UserSteppingAction(const G4Step *aStep)
{
#if defined (G4ANALYSIS_USE_ROOT)
  
  Double_t aStepLength = (aStep->GetStepLength() / cm); // in [cm]
  // Double_t aTotalEnergyLoss = (aStep->GetTotalEnergyDeposit() / MeV); // in [MeV]
  Double_t aTotalEnergyLoss = -(aStep->GetDeltaEnergy() / MeV); // in [MeV]
  Double_t aTotalEnergyLossPerLengthUnit = -1.0; // will be in [MeV/cm]
  Double_t aWeight = aStep->GetTrack()->GetWeight();
  
  if (hStepLength) hStepLength->Fill(aStepLength, aWeight); // Step Length
  
  if (hStepTotELoss) hStepTotELoss->Fill(aTotalEnergyLoss, aWeight); // Step Total Energy Loss
  
  if (aStepLength != 0.0) aTotalEnergyLossPerLengthUnit = aTotalEnergyLoss / aStepLength;
  else aTotalEnergyLossPerLengthUnit = aTotalEnergyLoss / 1.0e-7; // in [MeV/cm]
  if (hTotELossNorm) hTotELossNorm->Fill(aTotalEnergyLossPerLengthUnit, aWeight); // Total Energy Loss per Step Length Unit
  
#endif /* defined (G4ANALYSIS_USE_ROOT) */
  
  OnceAWhileDoIt();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void IcaAnalysisExample::OnceAWhileDoIt(const G4bool DoItNow)
{
  time_t Now = time(0); // get the current time (measured in seconds)
  if ( (!DoItNow) && (LastDoItTime > (Now - 10)) ) return; // every 10 seconds
  LastDoItTime = Now;
  
#if defined (G4ANALYSIS_USE_ROOT)
  
  TVirtualPad *CurrentPad = gPad;
  
  const char OnceAWhileCanvas[] = "IcaAnalysisExampleCanvas";
  
  TCanvas *c = (TCanvas *)gROOT->GetListOfCanvases()->FindObject(OnceAWhileCanvas);
  
  if ( DoItNow && (!c) && ((G4RunManager::GetRunManager())->GetVerboseLevel() > 1) )
    {
      // first, try to make sure we do not run in batch mode
      if (!gApplication)
	new TApplication("Application", ((int *)0), ((char **)0));
      // then, create the TCanvas
      c = new TCanvas(OnceAWhileCanvas, OnceAWhileCanvas);
      if (c) {
	c->Divide(2,2);
	c->cd(1); if (hStepLength) hStepLength->Draw();
	c->cd(2); if (hStepTotELoss) hStepTotELoss->Draw();
	c->cd(3); if (hOPWaveLength) hOPWaveLength->Draw();
	c->cd(4); if (hTotELossNorm) hTotELossNorm->Draw();
	c->cd(0);
      }
    }
  
  if (c) {
    //c->cd(0);
    c->Draw();
    c->Update();
  }
  
  if (CurrentPad) CurrentPad->cd();
  
  if (gSystem) gSystem->ProcessEvents();
  
#endif /* defined (G4ANALYSIS_USE_ROOT) */
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

// end of file IcaAnalysisExample.cc by Jacek M. Holeczek
