//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * The following disclaimer summarizes all the specific disclaimers *
// * of contributors to this software. The specific disclaimers,which *
// * govern, are listed with their locations in:                      *
// *   http://cern.ch/geant4/license                                  *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * This  code  implementation is the  intellectual property  of the *
// * GEANT4 collaboration.                                            *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//
// $Id: IcaPhysicsListMessenger.cc,v 1.2 2001/07/11 09:57:24 gunter Exp $
// GEANT4 tag $Name: geant4-04-00 $
//
// 

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

#include "IcaPhysicsListMessenger.hh"

#include "IcaPhysicsList.hh"
#include "G4UIcmdWithADoubleAndUnit.hh"
#include "G4UIcmdWithAString.hh"
#include "G4UIcmdWithAnInteger.hh"
#include "G4UIcmdWithADouble.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

IcaPhysicsListMessenger::IcaPhysicsListMessenger(IcaPhysicsList * List)
:IcaList(List)
{
  CutGCmd = new G4UIcmdWithADoubleAndUnit("/Simulation/SetGammaRangeCut",this);
  CutGCmd->SetGuidance("Set cut value by RANGE for gamma.");
  CutGCmd->SetParameterName("range",false);
  CutGCmd->SetDefaultValue(1.);
  CutGCmd->SetDefaultUnit("mm");
  CutGCmd->AvailableForStates(G4State_Idle);

  CutECmd = new G4UIcmdWithADoubleAndUnit("/Simulation/SetElectronRangeCut",this);
  CutECmd->SetGuidance("Set cut values by RANGE for e- e+.");
  CutECmd->SetParameterName("range",false);
  CutECmd->SetDefaultValue(1.);
  CutECmd->SetDefaultUnit("mm");
  CutECmd->AvailableForStates(G4State_Idle);

  CutMCmd = new G4UIcmdWithADoubleAndUnit("/Simulation/SetMuonRangeCut",this);
  CutMCmd->SetGuidance("Set cut values by RANGE for mu- mu+.");
  CutMCmd->SetParameterName("range",false);
  CutMCmd->SetDefaultValue(1.);
  CutMCmd->SetDefaultUnit("mm");
  CutMCmd->AvailableForStates(G4State_Idle);

  CutPCmd = new G4UIcmdWithADoubleAndUnit("/Simulation/SetProtonRangeCut",this);
  CutPCmd->SetGuidance("Set cut values by RANGE for proton and anti_proton.");
  CutPCmd->SetParameterName("range",false);
  CutPCmd->SetDefaultValue(1.);
  CutPCmd->SetDefaultUnit("mm");
  CutPCmd->AvailableForStates(G4State_Idle);

  CutOCmd = new G4UIcmdWithADoubleAndUnit("/Simulation/SetOpticalPhotonRangeCut",this);
  CutOCmd->SetGuidance("Set cut value by RANGE for optical photon.");
  CutOCmd->SetParameterName("range",false);
  CutOCmd->SetDefaultValue(1.);
  CutOCmd->SetDefaultUnit("mm");
  CutOCmd->AvailableForStates(G4State_Idle);

  ECmd = new G4UIcmdWithADoubleAndUnit("/Simulation/SetEnergyCut",this);
  ECmd->SetGuidance("Set cut values by ENERGY for all particles.");
  ECmd->SetParameterName("energy",false);
  ECmd->SetDefaultValue(10.);
  ECmd->SetDefaultUnit("keV");
  ECmd->AvailableForStates(G4State_Idle);

  RCmd = new G4UIcmdWithADoubleAndUnit("/Simulation/GetMuonRange",this);
  RCmd->SetGuidance("Display the RANGE of muons for the current material.");
  RCmd->SetParameterName("energy",false);
  RCmd->SetDefaultValue(1.);
  RCmd->SetDefaultUnit("GeV");
  RCmd->AvailableForStates(G4State_Idle);

  SetMaxStepCmd = new G4UIcmdWithADoubleAndUnit("/Simulation/SetMaxStepLength",this);
  SetMaxStepCmd->SetGuidance("Set max. step length for charged particles.");
  SetMaxStepCmd->SetParameterName("MaxStepLength",false);
  SetMaxStepCmd->SetDefaultValue(0.);
  // SetMaxStepCmd->SetRange("MaxStepLength>=0.");
  SetMaxStepCmd->SetUnitCategory("Length");
  SetMaxStepCmd->SetDefaultUnit("mm");
  SetMaxStepCmd->AvailableForStates(G4State_Idle);

  ResCutsCmd = new G4UIcmdWithADoubleAndUnit("/Simulation/RestoreDefaultCuts",this);
  ResCutsCmd->SetGuidance("Restore default cuts by RANGE for all particles.");
  ResCutsCmd->SetParameterName("DefaultCut",true);
  ResCutsCmd->SetDefaultValue(0.);
  ResCutsCmd->SetDefaultUnit("mm");
  ResCutsCmd->AvailableForStates(G4State_Idle);

  MaxPhotCmd = new G4UIcmdWithAnInteger("/Simulation/SetMaxNumOfCerenkovPhotonsPerStep",this);
  MaxPhotCmd->SetGuidance("Set max. num. of Cerenkov photons per step.");
  MaxPhotCmd->SetParameterName("MaxNumOfCerenkovPhotonsPerStep",false,false);
  // MaxPhotCmd->SetDefaultValue(1000);
  MaxPhotCmd->AvailableForStates(G4State_PreInit, G4State_Idle);

  ScinYieldFactorCmd = new G4UIcmdWithADouble("/Simulation/SetScintillationYieldFactor", this);
  ScinYieldFactorCmd->SetGuidance("Set scintillation yield factor.");
  ScinYieldFactorCmd->SetParameterName("ScintillationYieldFactor",false,false);
  // ScinYieldFactorCmd->SetDefaultValue(1.0);
  ScinYieldFactorCmd->AvailableForStates(G4State_PreInit, G4State_Idle);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

IcaPhysicsListMessenger::~IcaPhysicsListMessenger()
{
  delete CutGCmd;
  delete CutECmd;
  delete CutMCmd;
  delete CutPCmd;
  delete CutOCmd;
  delete ECmd;
  delete RCmd;

  delete SetMaxStepCmd;

  delete ResCutsCmd;

  delete MaxPhotCmd;

  delete ScinYieldFactorCmd;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....
  
void IcaPhysicsListMessenger::SetNewValue(G4UIcommand* command,G4String newValue)
{
  if(command == CutGCmd)
    { IcaList->SetGammaCut(CutGCmd->GetNewDoubleValue(newValue));}
  if(command == CutECmd)
    { IcaList->SetElectronCut(CutECmd->GetNewDoubleValue(newValue));}
  if(command == CutMCmd)
    { IcaList->SetMuonCut(CutMCmd->GetNewDoubleValue(newValue));}
  if(command == CutPCmd)
    { IcaList->SetProtonCut(CutPCmd->GetNewDoubleValue(newValue));}
  if(command == CutOCmd)
    { IcaList->SetOpticalPhotonCut(CutOCmd->GetNewDoubleValue(newValue));}
  if(command == ECmd)
    { IcaList->SetCutsByEnergy(ECmd->GetNewDoubleValue(newValue));}
  if(command == RCmd)
    { IcaList->GetRange(RCmd->GetNewDoubleValue(newValue));}
  if(command == SetMaxStepCmd)
    { IcaList->SetMaxStep(SetMaxStepCmd->GetNewDoubleValue(newValue));}
  if(command == ResCutsCmd)
    { IcaList->RestoreDefaultCuts(ResCutsCmd->GetNewDoubleValue(newValue));}
  if(command == MaxPhotCmd)
    { IcaList->SetMaxNumOfCerenkovPhotonsPerStep(MaxPhotCmd->GetNewIntValue(newValue));}
  if(command == ScinYieldFactorCmd)
    { IcaList->SetScintillationYieldFactor(ScinYieldFactorCmd->GetNewDoubleValue(newValue));}
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

// end of file IcaPhysicsListMessenger.cc by Jacek M. Holeczek
