//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * The following disclaimer summarizes all the specific disclaimers *
// * of contributors to this software. The specific disclaimers,which *
// * govern, are listed with their locations in:                      *
// *   http://cern.ch/geant4/license                                  *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * This  code  implementation is the  intellectual property  of the *
// * GEANT4 collaboration.                                            *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//
// $Id: IcaPrimaryGeneratorAction.cc,v 1.3 2001/11/21 11:57:14 mverderi Exp $
// GEANT4 tag $Name: geant4-04-00 $
//
// 

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

#include "IcaPrimaryGeneratorAction.hh"

#include "IcaDetectorConstruction.hh"
#include "IcaPrimaryGeneratorActionMessenger.hh"

#include "IcaAnalysisManager.hh"

#include "G4Event.hh"
#include "G4GeneralParticleSource.hh"
#include "G4ParticleTable.hh"
#include "G4ParticleDefinition.hh"
#include "Randomize.hh"
#include "G4ios.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

IcaPrimaryGeneratorAction::IcaPrimaryGeneratorAction(
                                            IcaDetectorConstruction* IcaDC)
:IcaDetector(IcaDC)
{
  if (!IcaDC)
    {
      G4cerr << "Warning: IcaPrimaryGeneratorAction default constructor called !!!"
	     << G4endl;
    }
  
  particleGun  = new G4GeneralParticleSource();
  
  // default particle kinematic
  SetDefaultKinematic();
  
  //create a messenger for this class
  gunMessenger = new IcaPrimaryGeneratorActionMessenger(this);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

IcaPrimaryGeneratorAction::~IcaPrimaryGeneratorAction()
{
  delete particleGun;
  delete gunMessenger;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void IcaPrimaryGeneratorAction::GeneratePrimaries(G4Event* anEvent)
{
  //this function is called at the begining of event
  // 
  particleGun->GeneratePrimaryVertex(anEvent);

  // Histogramming
  if(gIcaAnalysisManager) gIcaAnalysisManager->GeneratePrimaries(anEvent);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void IcaPrimaryGeneratorAction::SetDefaultKinematic()
{
  // default particle gun kinematic
  // 
  // "/gps/source/clear"
  particleGun->ClearAll();
  // "/gps/source/multiplevertex false"
  particleGun->SetMultipleVertex(false);
  // "/gps/source/add 1"
  particleGun->AddaSource(1.0);
  // "/gps/particle mu-"
  G4ParticleDefinition* particle =
    (G4ParticleTable::GetParticleTable())->FindParticle("mu-");
  particleGun->GetCurrentSource()->SetParticleDefinition(particle);
  // "/gps/energy 1 GeV"
  particleGun->GetCurrentSource()->GetEneDist()->SetEnergyDisType("Mono");
  particleGun->GetCurrentSource()->GetEneDist()->SetMonoEnergy(1.*GeV);
  // "/gps/position 0 0 z0"
  G4double z0 = 0.0;
  if (IcaDetector) z0 = -1.0*(IcaDetector->GetWorldSizeZ());
  particleGun->GetCurrentSource()->GetPosDist()->SetPosDisType("Point");
  particleGun->GetCurrentSource()->GetPosDist()->SetCentreCoords(G4ThreeVector(0.0, 0.0, z0));
  // "/gps/direction 0 0 1"
  particleGun->GetCurrentSource()->GetAngDist()->SetAngDistType("planar");
  particleGun->GetCurrentSource()->GetAngDist()->SetParticleMomentumDirection(G4ThreeVector(0.,0.,1.));
}

// end of file IcaPrimaryGeneratorAction.cc by Jacek M. Holeczek
