//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * The following disclaimer summarizes all the specific disclaimers *
// * of contributors to this software. The specific disclaimers,which *
// * govern, are listed with their locations in:                      *
// *   http://cern.ch/geant4/license                                  *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * This  code  implementation is the  intellectual property  of the *
// * GEANT4 collaboration.                                            *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//
// $Id: IcaSteppingAction.cc,v 1.6 2002/01/09 17:24:20 ranjard Exp $
// GEANT4 tag $Name: geant4-04-00-patch-02 $
//

#include "IcaSteppingAction.hh"
#include "IcaSteppingActionMessenger.hh"

#include "IcaEventAction.hh"

#include "IcaAnalysisManager.hh"

#include "G4VVisManager.hh"
#include "G4Polyline.hh"
#include "G4Colour.hh"
#include "G4VisAttributes.hh"
#include "G4SteppingManager.hh"

IcaSteppingAction::IcaSteppingAction(IcaEventAction* IcaEA)
:drawFlag("all"), eventAction(IcaEA)
{
  if (!IcaEA)
    {
      G4cerr << "Warning: IcaSteppingAction default constructor called !!!"
	     << G4endl;
    }
  
  steppingMessenger = new IcaSteppingActionMessenger(this);
}

IcaSteppingAction::~IcaSteppingAction()
{
  delete steppingMessenger;
}

void IcaSteppingAction::UserSteppingAction(const G4Step *aStep)
{
  // Check what is to be drawn from IcaEventAction/IcaEventActionMessenger
  G4String drawTrksFlag = "none";
  if (eventAction) drawTrksFlag = eventAction->GetDrawFlag();
  
  if (drawTrksFlag=="custom") {
    // Check what is to be drawn from IcaSteppingAction/IcaSteppingActionMessenger
    if(drawFlag != "none") {
      G4VVisManager* pVVisManager = G4VVisManager::GetConcreteInstance();
      
      if (pVVisManager) {
	const G4SteppingManager* pSM = fpSteppingManager;
	G4Polyline polyline;
	
	G4double charge = pSM->GetTrack()->GetDefinition()->GetPDGCharge();
	G4Colour colour;
	if      (charge < 0.) colour = G4Colour(1., 0., 0.);
	else if (charge > 0.) colour = G4Colour(0., 0., 1.);
	else 	          colour = G4Colour(0., 1., 0.);
	
	if ( (drawFlag == "all") ||
	     ((charge != 0.0) && (drawFlag == "charged")) ||
	     ((charge == 0.0) && (drawFlag == "neutral")) ) {
	  G4VisAttributes attribs(colour);
	  polyline.SetVisAttributes(attribs);
	  polyline.push_back(pSM->GetStep()->GetPreStepPoint()->GetPosition());
	  polyline.push_back(pSM->GetStep()->GetPostStepPoint()->GetPosition());
	  pVVisManager -> Draw(polyline);
	}
      }
    }
  }
  
  // Histogramming
  if (gIcaAnalysisManager) gIcaAnalysisManager->UserSteppingAction(aStep);
}

// end of file IcaSteppingAction.cc by Jacek M. Holeczek
