//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//

#include "IcaTrackingAction.hh"
#include "IcaTrackingActionMessenger.hh"

#include "IcaAnalysisManager.hh"

#include "G4TrackingManager.hh"
#include "G4Track.hh"
#include "G4TrackStatus.hh"
#include "G4Trajectory.hh"

IcaTrackingAction::IcaTrackingAction()
{
  // new messenger
  theMessenger = new IcaTrackingActionMessenger(this);
  
  // messenger defaults
  StoreTrajectoriesOfSecondariesFlag = true;
}

IcaTrackingAction::~IcaTrackingAction()
{
  delete theMessenger;
}

void IcaTrackingAction::PreUserTrackingAction(const G4Track* aTrack)
{
  // if !StoreTrajectoriesOfSecondariesFlag, create trajectory only for primaries
  if ( (StoreTrajectoriesOfSecondariesFlag) || (aTrack->GetParentID()==0) )
    { fpTrackingManager->SetStoreTrajectory(true); }
  else
    { fpTrackingManager->SetStoreTrajectory(false); }
  
  // Histogramming
  if (gIcaAnalysisManager) gIcaAnalysisManager->PreUserTrackingAction(aTrack);
}

void IcaTrackingAction::PostUserTrackingAction(const G4Track* aTrack)
{
  G4String particleName = aTrack->GetDefinition()->GetParticleName();
  G4TrackStatus status = aTrack->GetTrackStatus();
  
  // Get rid of secondary optical photons
  if ( (particleName == "opticalphoton") && (aTrack->GetParentID()!=0) )
    status = fKillTrackAndSecondaries;
  
  // Histogramming
  if (gIcaAnalysisManager) gIcaAnalysisManager->PostUserTrackingAction(aTrack, &status);
  
  // Set the final track status (delete the trajectory, if required)
  if (status == fKillTrackAndSecondaries) {
#if 0
    fpTrackingManager->SetStoreTrajectory(false);
#else // 0 or 1
    G4VTrajectory *fpTrajectory = fpTrackingManager->GimmeTrajectory();
    if (fpTrajectory) {
      delete fpTrajectory;
      fpTrackingManager->SetTrajectory( ((G4VTrajectory *)0) );
    }
    fpTrackingManager->SetStoreTrajectory(false);
    fpTrackingManager->EventAborted(); // sets fKillTrackAndSecondaries
#endif // 0 or 1
  }
#if 0
  fpTrackingManager->SetTrackStatus(status);
#endif // 0 or 1
}

// end of file IcaTrackingAction.cc by Jacek M. Holeczek
